@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2019 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2021 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -214,18 +214,23 @@ NOEXPORT int verify_callback(int preverify_ok, X509_STORE_CTX *callback_ctx) {
         s_log(LOG_INFO, "Certificate verification disabled");
         return 1; /* accept */
     }
-    if(verify_checks(c, preverify_ok, callback_ctx)) {
-        if(!SSL_SESSION_set_ex_data(SSL_get_session(ssl),
-                index_session_authenticated, (void *)(-1))) {
-            sslerror("SSL_SESSION_set_ex_data");
-            return 0; /* reject */
-        }
+    if(verify_checks(c, preverify_ok, callback_ctx))
         return 1; /* accept */
-    }
     if(c->opt->option.client || c->opt->protocol)
         return 0; /* reject */
-    if(c->opt->redirect_addr.names)
+    if(c->opt->redirect_addr.names) {
+        SSL_SESSION *sess=SSL_get1_session(c->ssl);
+        if(!sess)
+            return 0; /* reject */
+        if(!SSL_SESSION_set_ex_data(sess,
+                index_session_authenticated, NULL)) {
+            sslerror("SSL_SESSION_set_ex_data");
+            SSL_SESSION_free(sess);
+            return 0; /* reject */
+        }
+        SSL_SESSION_free(sess);
         return 1; /* accept */
+    }
     return 0; /* reject */
 }
 
@@ -651,7 +656,7 @@ NOEXPORT OCSP_RESPONSE *ocsp_get_response(CLI *c,
 
     /* OCSP protocol communication loop */
     while(OCSP_sendreq_nbio(&resp, req_ctx)==-1) {
-        s_poll_init(c->fds);
+        s_poll_init(c->fds, 0);
         s_poll_add(c->fds, c->fd, BIO_should_read(bio), BIO_should_write(bio));
         switch(s_poll_wait(c->fds, c->opt->timeout_busy, 0)) {
         case -1:
